<?php
/* **************************************************************************** *

 * Database.php									*
 * @package	Core								*
 * @copyright:	Copyright (c) 2008 Martin Novák. All rights reserved		*
 * @license	GNU/AGPL http://www.gnu.org/licenses/agpl.html			*
 
 * Albireo is free software! You can redistribute it and/or modify it under	*
 * the terms of the GNU Affero General Public License as published		*
 * by the Free Software Foundation, either version 3 of the License, or		*
 * (at your option) any later version.						*

 *	This program is distributed in the hope that it will be useful,		*
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of		*
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the		*
 *	GNU Affero General Public License for more details.			*

 This script description:
 This class was developed for work with MySQL database. Interface works with
 design pattern Singleton. You can access an instance by calling method
 connection.
 This class does not support mysqli interface.
 Example: $number_of_news = Database::connection () -> getResult ( $query );

 Properties:
    none
 	
 Public Methods:
    resource connection ()	// contain an instance of class (Singleton design
    				   pattern)
    resource query ( $query )	// SQL queries for database
    array fetchall_row ( $query )
    				// returns two dimensional array as result of query
    				   (number indexes)
    array fetchall_assoc ( $query )
    				// return two demensional array as result of query
    				   (first dimension contains number index and second
    				   associative names of columns)
    char getResult ( $query )	// returns string as result of query
    array fetch_assoc ( $query )// returns associative array as resutl of query
    boolean is_connected ()	// returns boolean information of connection to
    				   the database
    boolean reconnect ()	// reconnect to the database
    boolean connect_to ()	// connect to the database
    
 * **************************************************************************** */

class Database {

    private $dbh;
    private static $instance = null;

    public static function connection ()
    {
    	if ( self::$instance == null ) {
    	    self::$instance = new Database ();
    	}
    	return self::$instance;
    } // end connection

    private function __construct ()
    {
    	$dbhost = Tools::baseSettings () -> getSetting ( "dbhost" );
    	$user = Tools::baseSettings () -> getSetting ( "user" );
    	$pass = Tools::baseSettings () -> getSetting ( "pass" );
    	$dbname = Tools::baseSettings () -> getSetting ( "dbname" );
    
	if ( $this -> dbh = @mysql_connect ( $dbhost, $user, $pass, true ) ) {
	    if ( !mysql_select_db ( $dbname,  $this -> dbh ) ) {
	    	$this -> dbh = null;
	    	Error::ticket () -> notice ( 'cor00009','in '. __FILE__ .' on line '. __LINE__, mysql_errno () .': '. mysql_error () );
	    }
	} else {
	    Error::ticket () -> notice ( 'cor00010','in '. __FILE__ .' on line '. __LINE__, mysql_errno () .': '. mysql_error () );
	}
    } // end __construct

    private function __clone ()
    {
    } // end __clone

    public function query ( $query ) {
	if ( $result = @mysql_query ( $query, $this -> dbh ) ) {
    	    return $result;
    	} else {
    	    Error::ticket () -> notice ( 'cor00011','in '.__FILE__.' on line '.__LINE__, $query.'<br />MySQL: '.mysql_errno ().'> '.mysql_error () );
	    return false;
    	}
    } // end query

    public function fetchall_row ( $query )
    {
    	if ( !$result = $this -> query ( $query ) ) {
    	    return false;
    	}
    	$retval = array ();
    	while ( $row = mysql_fetch_row ( $result ) ) {
   	    $retval [] = $row;
  	}
  	return $retval;
    } // end fetchall_row

    public function fetchall_assoc ( $query )
    {
    	if ( !$result = $this -> query ( $query ) ) {
    	    return false;
    	}
    	$retval = array ();
  	while ( $row = mysql_fetch_assoc ( $result ) ) {
   	    $retval [] = $row;
  	}
  	return $retval;
    } // end fetchall_assoc

    public function getResult ( $query )
    {
	if ( !$result = $this -> query ( $query ) ) { // ověří úspěšné provedení dotazu na databázi
    	    return false;
    	}
	$num_rows = mysql_num_rows ( $result );
	if ( $num_rows == 1 ) {
	    $current_result = mysql_fetch_row ( $result );
	    return current ( $current_result );
	} else {
	    return false;
	}
    } // end getResult

    public function fetch_assoc ( $query )
    {
	if ( !$result = $this -> query ( $query ) ) {
    	    return false;
    	}
	return mysql_fetch_assoc ( $result );
    } // end fetch_assoc

    public function backup_table ( $table )
    {
    	$new_row = "";
    	@set_time_limit ( 0 );
    	$rows = "DROP TABLE IF EXISTS $table;\n";
    	$create = $this -> fetch_assoc ( "SHOW CREATE TABLE $table;" );
    	$rows .= $create [ "Create Table" ] .";\n";
    	$contents = $this -> fetchall_row ( "SELECT * FROM $table;" );
    	if ( is_array ( $contents ) ) {
    	    foreach ( $contents as $row ) {
    	    	$new_row = "INSERT INTO $table VALUES ( ";
    	    	foreach ( $row as $value ) {
    	    	    if ( is_numeric ( $value ) ) {
    	    	    	$new_row .= "$value, ";
    	    	    } else {
    	    	    	$new_row .= "'$value', ";
    	    	    }
    	    	}
    	    	$rows .= substr ( $new_row, 0, -2 ) ." );\n";
    	    }
    	}
    	return $rows;
    } // end backup_table

    public function backup_database ()
    {
    	$rows = "";
    	@set_time_limit ( 0 );

    	$tables = $this -> fetchall_row ( "SHOW TABLES;" );

    	foreach ( $tables as $table ) {
    	    $rows .= $this -> backup_table ( $table [ 0 ] );
    	}

	return $rows;
    } // end backup_database

    public function sql_from_file ( $path_to_file, $ignore = true ) // if is ignore true, the function will ignore errors during process
    {
    	$result = array ( "errors" => 0, "ok" => 0 );

	@set_time_limit ( 0 );

	if ( !( $handle = @fopen ( $path_to_file, 'r' ) ) ) {
	    return false;
	}

	$sql = '';
	while ( !feof ( $handle ) ) {
	    $line = fgets ( $handle );
	    if ( substr ( ltrim ( $line ), 0, 2 ) !== '--' ) {
	    	$sql .= $line;
	    	if ( substr ( rtrim ( $line ), -1 ) === ';' ) {
		    if ( @mysql_query ( $sql, $this -> dbh ) ) {
		    	$result [ "ok" ] = $result [ "ok" ] + 1;
		    } elseif ( $ignore == false ) {
		    	return false;
		    } else {
    	    	    	$result [ "errors" ] = $result [ "errors" ] + 1;
    	    	    }
		    $sql = '';
		}
	    }
	}
	fclose ( $handle );

    	return $result;
    } // end sql_from_file

    public function is_connected ()
    {
    	return is_resource ( $this -> dbh );
    } // end

    public function reconnect ()
    {
    	if ( self::$instance = new Database () ) {
    	    return true;
    	} else {
    	    return false;
    	}
    } // end reconnect

    public function connect_to ( $dbhost, $user, $pass, $dbname )
    {
	if ( $this -> dbh = @mysql_connect ( $dbhost, $user, $pass ) ) {
	    if ( !mysql_select_db ( $dbname,  $this -> dbh ) ) {
	    	$this -> dbh = null;
	    	Error::ticket () -> notice ( 'cor00009','in '. __FILE__ .' on line '. __LINE__, mysql_errno () .': '. mysql_error () );
	    } else {
	    	return true;
	    }
	} else {
	    Error::ticket () -> notice ( 'cor00010','in '. __FILE__ .' on line '. __LINE__, mysql_errno () .': '. mysql_error () );
	}
    } // end connect_to

}
?>