<?php
/* **************************************************************************** *

 * PackageManager.inc.php							*
 * @package	Core								*
 * @copyright:	Copyright (c) 2008 Martin Novák. All rights reserved		*
 * @license	GNU/AGPL http://www.gnu.org/licenses/agpl.html			*
 
 * Albireo is free software! You can redistribute it and/or modify it under	*
 * the terms of the GNU Affero General Public License as published		*
 * by the Free Software Foundation, either version 3 of the License, or		*
 * (at your option) any later version.						*

 *	This program is distributed in the hope that it will be useful,		*
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of		*
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the		*
 *	GNU Affero General Public License for more details.			*

 This script description:
 Class for managing articles of Albireo.

 Properties:
    array $page			// contains changes for array $page in ScriptPage

 Public Methods:
    array download ( $url, $package_name = "none" )
    				// download file from target url
    array installPackage ( $package_name = "none" )
    				// install files from downloaded package
    
 * **************************************************************************** */

class PackageManager {

    private $package_name;
    private $issue = array (
    		"errors" => 0,
    		"installed" => 0,
    		"ok messages" => "",
    		"error messages" => "" );

    public function __construct ()
    {
    } // end __construct

    public function __destruct ()
    {
    	$this -> removeAllTemporary ();
    } // end __destruct

    public function download ( $url, $package_name = "none" )
    {

	$this -> package_name = ( $package_name == "none" ) ? substr ( strrchr ( $url, "/" ), 1 ) : $package_name;

	// manage existence of folder for temporary files ////////////////////////////
	if ( !file_exists ( "temporary" ) ) {
	    if ( !@mkdir ( "temporary" ) ) {
	    	$this -> issue [ "errors" ] = $this -> issue [ "errors" ] + 1;
	    	$this -> issue [ "error messages" ] = '<p><strong class="warning">'. _( "Fatal Error - directory temporary cannot be created!" ) .'</strong> - write: '. substr ( sprintf ( '%o', fileperms ( '.' ) ), -4 ) .'</p>';
	    	return $this -> issue;
	    }
    	}
	chdir ( "temporary" );
	//////////////////////////////////////////////////////////////////////////////

	// download file and save it into variable ///////////////////////////////////
    	if ( !$content = @file_get_contents ( $url ) ) {
	    $this -> issue [ "errors" ] = $this -> issue [ "errors" ] + 1;
	    $this -> issue [ "error messages" ] = '<p><strong class="warning">'. _( "Fatal Error - file does not exist:" ) .' '. $url .'</strong></p>';
	    return $this -> issue;
	}
	//////////////////////////////////////////////////////////////////////////////

	// save file on drive ////////////////////////////////////////////////////////
	if ( !file_put_contents ( $this -> package_name, $content ) ) {
    	    $this -> issue [ "errors" ] = $this -> issue [ "errors" ] + 1;
	    $this -> issue [ "error messages" ] = '<p><strong class="warning">'. _( "Fatal Error - downloaded file cannot be saved!" ) .'</strong> - write: '. substr ( sprintf ( '%o', fileperms ( '.' ) ), -4 ) .'</p>';
	    return $this -> issue;
    	}
    	//////////////////////////////////////////////////////////////////////////////
    	chdir ( ".." );
    	return $this -> issue;
	
    } // end download

    private function extractZip ( $package_name )
    {

    	$zip = new ZipArchive;
        if ( @$zip -> open ( $package_name ) === true ) {
            $zip -> extractTo ( "." );
            $zip -> close ();
        } else {
            $this -> issue [ "errors" ] = $this -> issue [ "errors" ] + 1;
	    $this -> issue [ "error messages" ] = '<p><strong class="warning">'. _( "Fatal Error - downloaded zip package cannot be opened!" ) .'</strong></p>';
        }

    } // end extractZip

    public function installPackage ( $package_name = "none" )
    {
    
    	chdir ( "temporary" );

	$this -> package_name = ( $package_name == "none" ) ? $this -> package_name : $package_name;

	if ( substr ( strrchr ( $this -> package_name, "." ), 1 ) ) {
	    $this -> extractZip ( $this -> package_name ); // extract ziped package
	}

	chdir ( ".." );

	if ( $this -> issue [ "errors" ] > 0 ) {
	    return $this -> issue;
	}

	$this -> issue [ "ok messages" ] .= '<ul class="green">';
	$this -> issue [ "error messages" ] .= '<ul class="warning">';
	
	if ( file_exists ( "./temporary/package/" ) ) {
	    $this -> installDir ( "./temporary/package/", "./" );
	} else {
	    unlink ( "./temporary/". $this -> package_name );
	    $this -> installDir ( "./temporary/", "./" );
	}

	$this -> issue [ "ok messages" ] .= '</ul>';
	$this -> issue [ "error messages" ] .= '</ul>';

	return $this -> issue;

	
    } // end install

    private function installDir ( $package_directory, $install_directory )
    {

    	$folder = opendir ( $package_directory );

    	while ( $file = readdir ( $folder ) ) {

	    if ( is_dir ( $package_directory . $file ) ) {

	    	// installation in case of founding subfolder - recursion ////////////////////
	    	if ( $file != "." && $file != ".."  ) {
	    	    if ( !file_exists ( $install_directory . $file ) ) {
	    		if ( !@mkdir ( $install_directory . $file ) ) {
	    			$this -> issue [ "errors" ] = $this -> issue [ "errors" ] + 1;
				$this -> issue [ "error messages" ] .= '<li>'. _( "error - folder does not exist and cannot be created:" ). ' '. $install_directory . $file .'</li>';
	    		} else {
	    		    $this -> installDir ( $package_directory ."$file/", $install_directory ."$file/" );
	    		}
		    } else {
		    	$this -> installDir ( $package_directory ."$file/", $install_directory ."$file/" );
		    }
	    	}
	    	//////////////////////////////////////////////////////////////////////////////
	    	
	    } else {
	    	// make copy of file - install it ////////////////////////////////////////////
	    	if ( @copy ( $package_directory . $file, $install_directory . $file ) ) {
	    	    $this -> issue [ "installed" ] = $this -> issue [ "installed" ] + 1;
		    $this -> issue [ "ok messages" ] .= '<li>'. _( "file successfully installed:" ). ' '. $install_directory . $file ."</li>\n";
	    	} else {
	    	    $this -> issue [ "errors" ] = $this -> issue [ "errors" ] + 1;
	    	    $this -> issue [ "error messages" ] .= '<li>'._( "error - file cannot be installed:" ). ' '. $install_directory . $file ."</li>\n";
	    	}
	    	//////////////////////////////////////////////////////////////////////////////
	    	
	    }
	
	}

	closedir ( $folder );
	
    } // end installDir
    
    private function removeAllTemporary ()
    {
    	if ( file_exists ( "temporary" ) ) {
    	    chdir ( "temporary" );
    	    $this -> removeDir ( "./" );
    	    chdir ( ".." );
    	}
    	
    } // end removeAllTemporary

    private function removeDir ( $directory )
    {
    	$folder = opendir ( $directory );

    	while ( $file = readdir ( $folder ) ) {

	    if ( is_dir ( $directory . $file ) ) {

	    	// deleting in case of founding subfolder - recursion ////////////////////////
	    	if ( $file != "." && $file != ".."  ) {
		    $this -> removeDir ( $directory ."$file/" );
		    if ( !( @rmdir ( $directory ."$file/" ) ) ) {
		    	$this -> issue [ "errors" ] = $this -> issue [ "errors" ] + 1;
	    	    	$this -> issue [ "error messages" ] .= '<li>'._( "error - folder cannot be deleted:" ). ' '. $directory . $file .', '. _( "write:" ) .' '. substr ( sprintf ( '%o', fileperms ( $directory . $file ) ), -4 ) ."</li>\n";
		    }
	    	}
	    	//////////////////////////////////////////////////////////////////////////////
	    	
	    } else {
	    
	    	// delete temporary file /////////////////////////////////////////////////////
	    	if ( !( @unlink ( $directory . $file ) ) ) {
	    	    $this -> issue [ "errors" ] = $this -> issue [ "errors" ] + 1;
	    	    $this -> issue [ "error messages" ] .= '<li>'._( "error - file cannot be deleted:" ). ' '. $directory . $file .', '. _( "write:" ) .' '. substr ( sprintf ( '%o', fileperms ( $directory . $file ) ), -4 ) ."</li>\n";
	    	}
	    	//////////////////////////////////////////////////////////////////////////////
	    	
	    }
	
	}

	closedir ( $folder );
    } // removeDir

}

?>