﻿<?php
/* **************************************************************************** *

 * Publisher.inc.php								*
 * @version: 	1.0.0								*
 * @package	Core								*
 * @copyright:	Copyright (c) 2007 - 2008 Ragnar.cz. All rights reserved	*
 * @license	GNU/GPL http://www.gnu.org/copyleft/gpl.html			*
 
 * Albireo is free software! This file may have been modified pursuant to the	*
 * GNU General Public License, and as distributed it includes or is derivative	*
 * of works licensed under the GNU General Public License or other free or open	*
 * source software licenses.							*

 This script description:
 This script is used to parse XML strings or files.


 Properties:
   array $data_array		 // all elemnent and data are stored here.
   				    Starting element is signed by dot ( . ),
   				    endind element by slash ( / ), data are
   				    not marked
   array $attr_array		// all attributes are stored here as array (it
   				   has same number as the element-owner e.g. if
   				   element is $this->data_array[5] then its
   				   attributes are here:
   				   $this->attr_array[5]["whatever"]
   string $parse_only		// use when you want to parse olny content of
   				   some element
 Public Methods:
    parserStart ( $resource )	// is used to begin parsing file or string
    				   determined in $resource
 	
 * **************************************************************************** */

class Publisher {
    public $data_array;
    public $attr_array;
    private $opened;
    private $i;
    public  $parse_only;
     
    public function parserStart ( $resource )
    {
	    $this -> data_array = null;
	    $this -> attr_array = null;
	    $this -> i = 0;
	    $this -> opened = 0;
	    $analysis_handler = xml_parser_create ();
	    xml_set_object ( $analysis_handler, $this );
	    xml_parser_set_option($analysis_handler,XML_OPTION_CASE_FOLDING,0);

	    if ( empty ( $this -> parse_only ) ) {
    	    xml_set_element_handler ( $analysis_handler, "parseAllToArrayBegin", "parseAllToArrayEnd" );
	        xml_set_character_data_handler ( $analysis_handler, "parseAllToArrayData" );
	    } else {
    	    xml_set_element_handler ( $analysis_handler, "parseOnlyBegin", "parseOnlyEnd" );
	        xml_set_character_data_handler ( $analysis_handler, "parseOnlyData" );
	    }

	    $xml = $this -> loadData ( $resource );
	    if ( !xml_parse ( $analysis_handler, $xml ) ) {
    	    $this->xmlError($analysis_handler);
	    }   
    } // end parserStart
   
    protected function loadData ( $resource )
    {
	    if ( file_exists ( $resource ) ) {
    	    return file_get_contents ( $resource, 0);
	    } else {
    	    return $resource;
    	}
    } // end loadData
	
    protected function parseAllToArrayBegin ( $analysis_handler, $tag_name, $attributes )
    {
	    $this -> data_array [ $this -> i ] = "." . $tag_name;
	    foreach ( $attributes as $name => $value ) {
    	    $this -> attr_array [ $this -> i ] [ $name ] = $value;
    	}
    	$this -> i++;
    } // end parseAllToArrayBegin
	
    protected function parseAllToArrayEnd ( $analysis_handler, $tag_name )
    {
     	$this -> data_array [ $this -> i ] = "/". $tag_name;
    	$this -> i++;
    } // end parseAllToArrayEnd
	
    protected function parseAllToArrayData ( $analysis_handler, $data )
    {
	    $this -> data_array [ $this -> i ] = trim ( $data );
	    $this -> i++;
    } // end parseAllToArrayData
	
    protected function parseOnlyBegin ( $analysis_handler, $tag_name, $attributes )
    {   
	    if ( $tag_name == $this -> parse_only ) {
    	    $this -> opened++;
    	}
    	if ( $this -> opened > 0 ) {
	        $this -> data_array [ $this -> i ] = ".". $tag_name;
	        foreach ( $attributes as $name => $value ) {
		        $this -> attr_array [ $this -> i ] [ $name ] = $value;
	        }
	    $this -> i++;
	    }
    } // end parseOnlyBegin
	
    protected function parseOnlyEnd ( $analysis_handler, $tag_name )
    {
	    if ( $this -> opened > 0 ) {
    	    $this -> data_array [ $this -> i ] = "/". $tag_name;
	        $this -> i++;
	    }
	    if ( $tag_name == $this -> parse_only ) {
    	    $this -> opened--;
    	}
    } // end parseOnlyEnd
	
    protected function parseOnlyData ( $analysis_handler, $data )
    {
    	if ( $this -> opened > 0 ) {
	        $this -> data_array [ $this -> i ] = trim ( $data );
	        $this -> i++;
	    }
    } // end parseOnlyData
	
    protected function xmlError ( $analysis_handler )
    {
        return false;
    } // end xmlError
}
?>