<?php
/* **************************************************************************** *

 * ScriptPage.inc.php								*
 * @version: 	1.0.0								*
 * @package	Core								*
 * @copyright:	Copyright (c) 2007 - 2008 Ragnar.cz. All rights reserved	*
 * @license	GNU/GPL http://www.gnu.org/copyleft/gpl.html			*
 
 * Albireo is free software! This file may have been modified pursuant to the	*
 * GNU General Public License, and as distributed it includes or is derivative	*
 * of works licensed under the GNU General Public License or other free or open	*
 * source software licenses.							*

 This script description:
 Class ScriptPage is called by class Page for managing pages combinating MySQL
 database and dynamic php scripts.

 Properties:
    none
 	
 Public Methods:
    string getAccess ()		// show access of requested page
    string getType ()		// show name of class
    string getText ()		// show text of requested page
    string getTitle ()		// show title of requested page
    string getEditorial ()	// show editorial of requested page
    int getAuthor ()		// show id of user who created this page
    string getStatus ()		// show status of page
    int getDirectory ()		// show id of directory
    datetime getPublished ()	// show time of publishing

 * **************************************************************************** */

class ScriptPage implements PageInterface {

    private $page;

    public function __construct ( $getPage = "null" )
    {

	$getPage = ( $getPage == "null" ) ? Tools::getPage () : $getPage;
	
	$id_file = Database::connection () -> getResult ( "SELECT id FROM header WHERE file='$getPage';" );

	$this -> page = Database::connection () -> fetch_assoc ( "SELECT * FROM header, content WHERE header.id=$id_file AND content.id_header=$id_file;" ); // private variable page will contain associative array corresponding to database tables header and content

	// access checking ///////////////////////////////////////////////////////////
	if ( $this -> page [ "directory" ] == 0 ) { // page is in no directory, only article access will be checked
	    if ( !SecurityTools::check_read ( $this -> page [ "access" ], $this -> page [ "author" ], $this -> page [ "groupid" ] ) ) {
	    	if ( $this -> page [ "directory" ] == 1 || Database::connection () -> getResult ( "SELECT parent FROM directory WHERE id=". $this -> page [ "directory" ] .";" ) == 1 ) {
	    	    header ( "Location:". Localization::session () -> getFile ( "cor-log-in" ) ."?loginpage=". Tools::getPage () );
	    	} else {
	    	    Error::ticket () -> notice ( 'cor00028', 'in '. __FILE__ .' on line '. __LINE__ , $getPage ." ". User::session () -> getNick () );
	    	}
	    }
	} else { // system will check article and directory access
	    $parent_dir = Database::connection () -> fetch_assoc ( "SELECT author, groupid, access FROM directory WHERE id=". $this -> page [ "directory" ] .";" );
	    if ( !SecurityTools::check_read ( $this -> page [ "access" ], $this -> page [ "author" ], $this -> page [ "groupid" ] ) ||
	    	 !SecurityTools::check_read ( $parent_dir [ "access" ], $parent_dir [ "author" ], $parent_dir [ "groupid" ] ) ) {
	    	if ( $this -> page [ "directory" ] == 1 || Database::connection () -> getResult ( "SELECT parent FROM directory WHERE id=". $this -> page [ "directory" ] .";" ) == 1 ) {
	    	    header ( "Location:". Localization::session () -> getFile ( "cor-log-in" ) ."?loginpage=". Tools::getPage () );
	    	} else {
	    	    Error::ticket () -> notice ( 'cor00028', 'in '. __FILE__ .' on line '. __LINE__ , $getPage ." ". User::session () -> getNick () );
	    	}
	    }
	}
	//////////////////////////////////////////////////////////////////////////////

	$base_directory = $this -> page [ "directory" ];
	$this -> page [ "status" ] = "web";
	while ( $base_directory != 0 ) {
	    if ( $base_directory == 1 ) {
	    	$this -> page [ "status" ] = "admin";
	    }
	    $base_directory = Database::connection () -> getResult ( "SELECT parent FROM directory WHERE id=$base_directory;" );
	}

	$script_file = Database::connection () -> getResult ( "SELECT script FROM scriptpages WHERE id_header=$id_file;" );
	// require requested php script //////////////////////////////////////////////
	if ( file_exists ( "$script_file.php" ) ) { // check if the file really exists
	    require_once ( "$script_file.php" );  // contains $page containning changes for $this -> page
	    if ( is_array ( $page ) ) { // check script security
	    	foreach ( $page as $key => $value ) {
	    	    $this -> page [ $key ] = $value;
	    	}
	    } else {
	    	Error::ticket () -> notice ( 'cor00020', 'in '. __FILE__ .'-'. __CLASS__ .' on line '. __LINE__, "$script_file.php" );
	    }
	} else {
	    Error::ticket () -> notice ( 'cor00021', 'in '. __FILE__ .'-'. __CLASS__ .' on line '. __LINE__, "$script_file.php" );
	}
	//////////////////////////////////////////////////////////////////////////////

    } // end __construct

    public function getType ()
    {
    	return $this -> page [ "type" ];
    } // end getType

    public function getAuthor ()
    {
    	return $this -> page [ "author" ];
    }
    
    public function getAccess ()
    {
    	return $this -> page [ "access" ];
    }

    public function getStatus ()
    {
    	return $this -> page [ "status" ];
    }

    public function getText ()
    {
    	if ( $this -> page [ "text" ] == "0" ) {
    	    return "";
    	} else {
    	    return $this -> page [ "text" ];
    	}
    }

    public function getTitle ()
    {
    	return $this -> page [ "title" ];
    }

    public function getEditorial ()
    {
    	return $this -> page [ "editorial" ];
    }

    public function getDirectory ()
    {
    	return $this -> page [ "directory" ];
    } // end getDirectory

    public function getPublished ()
    {
    	return $this -> page [ "published" ];
    } // end getPublished
    
}

?>