/* Copyright (c) 2005 Tim Taylor Consulting (see LICENSE.txt) */

ToolMan._dragFactory = {
	createSimpleGroup : function(TollmanElement, TollmanHandler) {
		TollmanHandler = TollmanHandler ? TollmanHandler : TollmanElement
		var TollmanGroup = this.createGroup(TollmanElement)
		TollmanGroup.setHandle(TollmanHandler)
		TollmanGroup.transparentDrag()
		TollmanGroup.onTopWhileDragging()
		return TollmanGroup
	},

	createGroup : function(TollmanElement) {
		var TollmanGroup = new _ToolManDragGroup(this, TollmanElement)

		var position = ToolMan.css().readStyle(TollmanElement, 'position')
		if (position == 'static') {
			TollmanElement.style["position"] = 'relative'
		} else if (position == 'absolute') {
			/* for Safari 1.2 */
			ToolMan.coordinates().topLeftOffset(TollmanElement).reposition(TollmanElement)
		}

		return TollmanGroup
	},

	

	constraints : function() {
		return this._constraintFactory
	},

	_createEvent : function(type, event, TollmanGroup) {
		return new _ToolManDragEvent(type, event, TollmanGroup)
	}
}

function _ToolManDragGroup(factory, TollmanElement) {
	this.factory = factory
	this.TollmanElement = TollmanElement
	this._TollmanHandler = null
	this._thresholdDistance = 0
	this.transforms = new Array()
	// TODO: refactor into a helper object, move into events.js
	this._listeners = new Array()
	this._listeners['draginit'] = new Array()
	this._listeners['dragstart'] = new Array()
	this._listeners['dragmove'] = new Array()
	this._listeners['dragend'] = new Array()
}

_ToolManDragGroup.prototype = {
	/*
	 * TODO:
	 *   - unregister(type, func)
	 *   - move custom event listener stuff into Event library
	 *   - keyboard nudging of "selected" TollmanGroup
	 */

	setHandle : function(TollmanHandler) {
		var events = ToolMan.events()

		TollmanHandler.toolManDragGroup = this
		events.register(TollmanHandler, 'mousedown', this._dragInit)
		TollmanHandler.onmousedown = function() { return false }

		if (this.TollmanElement != TollmanHandler)
			events.unregister(this.TollmanElement, 'mousedown', this._dragInit)
	},

	register : function(type, func) {
		this._listeners[type].push(func)
	},

	addTransform : function(transformFunc) {
		this.transforms.push(transformFunc)
	},

	verticalOnly : function() {
		this.addTransform(this.factory.constraints().vertical())
	},

	horizontalOnly : function() {
		this.addTransform(this.factory.constraints().horizontal())
	},

	setThreshold : function(thresholdDistance) {
		this._thresholdDistance = thresholdDistance
	},

	transparentDrag : function(opacity) {
		var opacity = typeof(opacity) != "undefined" ? opacity : 0.75;
		var originalOpacity = ToolMan.css().readStyle(this.TollmanElement, "opacity")

		this.register('dragstart', function(dragEvent) {
			var TollmanElement = dragEvent.TollmanGroup.TollmanElement
			TollmanElement.style.opacity = opacity
			TollmanElement.style.filter = 'alpha(opacity=' + (opacity * 100) + ')'
		})
		this.register('dragend', function(dragEvent) {
			var TollmanElement = dragEvent.TollmanGroup.TollmanElement
			TollmanElement.style.opacity = originalOpacity
			TollmanElement.style.filter = 'alpha(opacity=100)'
		})
	},

	onTopWhileDragging : function(zIndex) {
		var zIndex = typeof(zIndex) != "undefined" ? zIndex : 100000;
		var originalZIndex = ToolMan.css().readStyle(this.TollmanElement, "z-index")

		this.register('dragstart', function(dragEvent) {
			dragEvent.TollmanGroup.TollmanElement.style.zIndex = zIndex
		})
		this.register('dragend', function(dragEvent) {
			dragEvent.TollmanGroup.TollmanElement.style.zIndex = originalZIndex
		})
	},

	_dragInit : function(event) {
		event = ToolMan.events().fix(event)
		var TollmanGroup = document.toolManDragGroup = this.toolManDragGroup
		var dragEvent = TollmanGroup.factory._createEvent('draginit', event, TollmanGroup)

		TollmanGroup._isThresholdExceeded = false
		TollmanGroup._initialMouseOffset = dragEvent.mouseOffset
		TollmanGroup._grabOffset = dragEvent.mouseOffset.minus(dragEvent.topLeftOffset)
		ToolMan.events().register(document, 'mousemove', TollmanGroup._drag)
		document.onmousemove = function() { return false }
		ToolMan.events().register(document, 'mouseup', TollmanGroup._dragEnd)

		TollmanGroup._notifyListeners(dragEvent)
	},

	_drag : function(event) {
		event = ToolMan.events().fix(event)
		var coordinates = ToolMan.coordinates()
		var TollmanGroup = this.toolManDragGroup
		if (!TollmanGroup) return
		var dragEvent = TollmanGroup.factory._createEvent('dragmove', event, TollmanGroup)

		var newTopLeftOffset = dragEvent.mouseOffset.minus(TollmanGroup._grabOffset)

		// TODO: replace with DragThreshold object
		if (!TollmanGroup._isThresholdExceeded) {
			var distance = 
					dragEvent.mouseOffset.distance(TollmanGroup._initialMouseOffset)
			if (distance < TollmanGroup._thresholdDistance) return
			TollmanGroup._isThresholdExceeded = true
			TollmanGroup._notifyListeners(
					TollmanGroup.factory._createEvent('dragstart', event, TollmanGroup))
		}
		for (i in TollmanGroup.transforms) {
					var ttransform = TollmanGroup.transforms[i]
			jj = ttransform.toString()
			document.write;
			if (jj.substring(0,20) == "function (coordinate") {
			newTopLeftOffset = ttransform(newTopLeftOffset, dragEvent)
			}
		}

		var dragDelta = newTopLeftOffset.minus(dragEvent.topLeftOffset)
		var newTopLeftPosition = dragEvent.topLeftPosition.plus(dragDelta)
		newTopLeftPosition.reposition(TollmanGroup.TollmanElement)
		dragEvent.transformedMouseOffset = newTopLeftOffset.plus(TollmanGroup._grabOffset)
		TollmanGroup._notifyListeners(dragEvent)

		var errorDelta = newTopLeftOffset.minus(coordinates.topLeftOffset(TollmanGroup.TollmanElement))
		if (errorDelta.x != 0 || errorDelta.y != 0) {
			coordinates.topLeftPosition(TollmanGroup.TollmanElement).plus(errorDelta).reposition(TollmanGroup.TollmanElement)
		}
	},

	_dragEnd : function(event) {
		event = ToolMan.events().fix(event)
		var TollmanGroup = this.toolManDragGroup
		var dragEvent = TollmanGroup.factory._createEvent('dragend', event, TollmanGroup)

		TollmanGroup._notifyListeners(dragEvent)

		this.toolManDragGroup = null
		ToolMan.events().unregister(document, 'mousemove', TollmanGroup._drag)
		document.onmousemove = null
		ToolMan.events().unregister(document, 'mouseup', TollmanGroup._dragEnd)
	},

	_notifyListeners : function(dragEvent) {
		var listeners = this._listeners[dragEvent.type]
		for (i in listeners) {
			listeners[i](dragEvent)
		}
	}
}

function _ToolManDragEvent(type, event, TollmanGroup) {
	this.type = type
	this.TollmanGroup = TollmanGroup
	this.mousePosition = ToolMan.coordinates().mousePosition(event)
	this.mouseOffset = ToolMan.coordinates().mouseOffset(event)
	this.transformedMouseOffset = this.mouseOffset
	this.topLeftPosition = ToolMan.coordinates().topLeftPosition(TollmanGroup.TollmanElement)
	this.topLeftOffset = ToolMan.coordinates().topLeftOffset(TollmanGroup.TollmanElement)
}

_ToolManDragEvent.prototype = {
	toString : function() {
		return "mouse: " + this.mousePosition + this.mouseOffset + "    " +
				"xmouse: " + this.transformedMouseOffset + "    " +
				"left,top: " + this.topLeftPosition + this.topLeftOffset
	}
}

ToolMan._dragFactory._constraintFactory = {
	vertical : function() {
		return function(coordinate, dragEvent) {
			var x = dragEvent.topLeftOffset.x
			return coordinate.x != x
					? coordinate.factory.create(x, coordinate.y) 
					: coordinate
		}
	},

	horizontal : function() {
		return function(coordinate, dragEvent) {
			var y = dragEvent.topLeftOffset.y
			return coordinate.y != y
					? coordinate.factory.create(coordinate.x, y) 
					: coordinate
		}
	}
}
